import java.awt.*;
import java.awt.event.*;

/**
 * Visualisierung eines arithmetischen Baums
 * 
 * @author Owyn
 * @version 2007-05-31
 */
public class Vis extends Frame {

    public final static int REGULAR = 1; // Darstellung mit gleichen Abstnden zwischen Knoten
    public final static int BINARY = 2;  // Darstellung mit fortlaufend halbierten Abstnden

    Token tree;
    int spacing;
    int width;
    int height;
    
    // Farben
    Color colBG = new Color(235,245,255);
    Color colNum = new Color(0,50,200);
    Color colOp = new Color(200,0,0);
    Color colText = new Color(255,245,100);
    Color colLines = new Color(0,0,240);
    Color colBorder = new Color(0,0,150);

    /**
     * Erzeugt das Grafikfenster (unsichtbar).
     * 
     * @param t darzustellender Baum
     * @param s Spacing: Vis.REGULAR oder Vis.BINARY
     */
    Vis(Token t, int s) {
        tree = t;
        spacing = s;
        
        // Fenstergroesse: 1000x700 Pixel
        width = 1000;
        height = 700;
        setSize(width, height);
        
        // Ein Klick auf das "X" zum Schliessen soll Wirkung zeigen
        addWindowListener( new WindowAdapter() { 
            @Override 
            public void windowClosing (WindowEvent e) { System.exit(0); } 
        } ); 
        
    }
    
    /**
     * Zeichnet einen Baumknoten.
     * 
     * @param g Grafik-Kontext
     * @param x x-Position (Mittelpunkt des Knotens)
     * @param y y-Position (Mittelpunkt des Knotens)
     * @param node darzustellender Baumknoten
     */
    public void drawSingleNode(Graphics g, int x, int y, Token node) {
        String txt = "";
        if (node.type() == 'n') { // Number node
            g.setColor(colNum);
            txt += node.eval();
        } else { // Operator node
            g.setColor(colOp);
            txt += node.type();
        }
        int txtwidth = g.getFontMetrics().charsWidth(txt.toCharArray(), 0, txt.length()); // Determine width of text
        g.fillRect(x - txtwidth / 2 - 3, y - 10, txtwidth + 5, 20);
        g.setColor(colBorder);
        g.drawRect(x - txtwidth / 2 - 3, y - 10, txtwidth + 5, 20);
        g.setColor(colText);
        g.drawString(txt, x - txtwidth / 2, y + 5);
    }
    
    /**
     * Zeichnet einen Baum mit gleichmaessigem Abstand zwischen Knoten.
     * 
     * @param g Grafik-Kontext
     * @param t darzustellender Baum
     */
    public void drawTreeReg(Graphics g, Token t) {
        drawTreeReg(g, t, 0, 0);
    }
    
    /**
     * Zeichnet einen Baum mit gleichmaessigem Abstand zwischen Knoten.
     * 
     * @param g Grafik-Kontext
     * @param t darzustellender Baum
     * @param depth aktuelle Tiefenebene im Baum
     * @param xorig x-Position des Vaterknotens
     */
    private void drawTreeReg(Graphics g, Token t, int depth, int xorig) {
        int ypos = 50 + depth * 70;
        int xpos = width * t.ord() / (tree.nodes() + 1);
        if (depth > 0) {
            g.setColor(colLines);
            g.drawLine(xpos, ypos - 10, xorig, ypos - 60);
        }
        drawSingleNode(g, xpos, ypos, t);
        if (t.type() != 'n') {
            drawTreeReg(g, t.left(), depth + 1, xpos);
            drawTreeReg(g, t.right(), depth + 1, xpos);
        }
    }
    
    /**
     * Zeichnet einen Baum mit fortlaufend halbierten Abstaenden.
     * 
     * @param g Grafik-Kontext
     * @param t darzustellender Baum
     */
    public void drawTreeBin(Graphics g, Token t) {
        drawTreeBin(g, t, 0, "", 0);
    }

    /**
     * Zeichnet einen Baum mit fortlaufend halbierten Abstaenden.
     * 
     * @param g Grafik-Kontext
     * @param t darzustellender Baum
     * @param depth aktuelle Tiefenebene im Baum
     * @param history Pfad bestehend aus "l" und "r" von der Wurzel zum aktuellen Knoten
     * @param xorig x-Position des Vaterknotens
     */
    private void drawTreeBin(Graphics g, Token t, int depth, String history, int xorig) {
        int ypos = 50 + depth * 70;
        int xpos = width / 2;
        int xdelta = width / 4;
        for (int i = 0; i < history.length(); i++) {
            if (history.charAt(i) == 'l') xpos -= xdelta;
             else xpos += xdelta;
            xdelta /= 2;
        }
        if (depth > 0) {
            g.setColor(colLines);
            g.drawLine(xpos, ypos - 10, xorig, ypos - 60);
        }
        drawSingleNode(g, xpos, ypos, t);
        if (t.type() != 'n') {
            drawTreeBin(g, t.left(), depth + 1, history + "l", xpos);
            drawTreeBin(g, t.right(), depth + 1, history + "r", xpos);
        }
    }

    /**
     * Zeichnen des Baums und Anzeige des Grafikfensters
     * 
     * @param g Grafik-Kontext
     */
    public void paint(Graphics g) {
        
        // Fenster loeschen
        g.setColor(colBG);
        g.fillRect(0, 0, width, height);
        g.setFont(new Font("SansSerif", Font.BOLD, 12));

        if (spacing == REGULAR) {
            Order o = new Order(); // Zaehler
            tree.order(o); // Baum in Infix-Reihenfolge nummerieren
            drawTreeReg(g, tree);
        } else drawTreeBin(g, tree);
    }

}
